import React, {useMemo} from "react";
import {classes} from "../../Utils";

/**
 * Calendar component.
 */
export function Calendar({date, onDateSelected, locale, className, ...tableProps}: {
	date: Date;
	onDateSelected: (date: Date) => void;
	locale?: string;
} & React.TableHTMLAttributes<HTMLTableElement>): React.ReactElement
{
	locale = useMemo(() => (locale ?? "fr"), [locale]);

	const currentMonthHeader = useMemo(() => (
		<tr>
			{ // For each day of the week, showing its name.
				[1, 2, 3, 4, 5, 6, 7].map(day => {
					// Getting a date with the right day of the week.
					const dayOfWeek = new Date(1970, 0, 4 + day);

					return (
						<th key={day}>
							{(new Intl.DateTimeFormat(locale, {weekday: "short"})).format(dayOfWeek)}
						</th>
					);
				})
			}
		</tr>
	), [date]);

	const currentMonthTable = useMemo(() => {
		// Initialize weeks.
		const weeksRows = [];
		let currentWeek = [];

		// Get start date of the calendar.
		let currentDate = new Date(date);
		currentDate.setDate(1); // First day of the month.
		currentDate.setDate(currentDate.getDate() - (currentDate.getDay() - 1 + 7) % 7); // Searching the start of the first week of the current month.

		// Get last day of the calendar.
		const lastDate = new Date(date);
		lastDate.setMonth(lastDate.getMonth() + 1, 0); // Get the last day of the month.
		lastDate.setDate(lastDate.getDate() + (7 - lastDate.getDay())); // Searching the end of the last week of the current month.


		while (currentDate.getTime() <= lastDate.getTime() || weeksRows.length < 6)
		{ // While the current date is before or is the last day of the current view,
			// adding the current day to the current week.
			currentWeek.push(
				<Day key={`${currentDate.getFullYear()}-${currentDate.getMonth()}-${currentDate.getDate()}`}
				     date={new Date(currentDate)}
				     faded={date.getMonth() != currentDate.getMonth()}
				     selected={date.getTime() == currentDate.getTime()}
				     onClick={onDateSelected} />
			);

			// We're on sunday, adding the current week and creating a new one.
			if (currentDate.getDay() == 0)
			{ // The current week is ended, adding it and creating a new one.
				weeksRows.push(
					<tr key={`${currentDate.getFullYear()}-${currentDate.getMonth()}-${currentDate.getDate()}`}>
						{currentWeek}
					</tr>
				);
				currentWeek = []; // Reset the current week to a new one.
			}

			// Get the next day.
			currentDate.setDate(currentDate.getDate() + 1);
		}

		// Return generated weeks rows.
		return weeksRows;
	}, [date, onDateSelected]);

	return (
		<table className={classes("calendar", className)} {...tableProps}>
			<thead>
			{currentMonthHeader}
			</thead>
			<tbody>
			{currentMonthTable}
			</tbody>
		</table>
	);
}

/**
 * Calendar day component.
 */
function Day({date, onClick, faded, selected}: {
	date: Date;
	onClick?: (date: Date) => void;
	faded: boolean;
	selected: boolean;
}): React.ReactElement
{
	return (
		<td key={`${date.getFullYear()}-${date.getMonth()}-${date.getDate()}`}>
			<a className={`day${faded ? " faded" : ""}${selected ? " selected" : ""}`}
			   onClick={() => onClick?.(date)}>
				{date.getDate()}
			</a>
		</td>
	);
}
