import React, {useCallback, useMemo} from "react";
import {CaretLeft, CaretRight} from "@phosphor-icons/react";
import {Tooltip} from "../Floating/Tooltip";
import {Calendar} from "./Calendar";
import {classes} from "../../Utils";

/**
 * Datepicker component.
 */
export function Datepicker({date, onDateSelected, locale, className, ...divProps}: {
	date: Date;
	onDateSelected: (date: Date) => void;
	locale?: string;
} & React.HTMLAttributes<HTMLDivElement>): React.ReactElement
{
	locale = useMemo(() => (locale ?? "fr"), [locale]);

	// Get previous month name.
	const previousMonthName = useMemo(() => {
		// Copy the current date and get back one month earlier.
		const previousMonthDate = new Date(date);
		previousMonthDate.setMonth(previousMonthDate.getMonth() - 1);
		return (new Intl.DateTimeFormat(locale, { month: "long", year: "numeric" })).format(previousMonthDate);
	}, [date]);
	// Get next month name.
	const nextMonthName = useMemo(() => {
		// Copy the current date and go to one month later.
		const nextMonthDate = new Date(date);
		nextMonthDate.setMonth(nextMonthDate.getMonth() + 1);
		return (new Intl.DateTimeFormat(locale, { month: "long", year: "numeric" })).format(nextMonthDate);
	}, [date]);

	return (
		<div className={classes("datepicker", className)} {...divProps}>
			<div className={"year-month"}>
				{(new Intl.DateTimeFormat(locale, {month: "long", year: "numeric"})).format(date)}
			</div>

			<Calendar date={date} onDateSelected={onDateSelected} />

			<Tooltip content={previousMonthName}>
				<button type={"button"} className={"previous-month"} onClick={
					useCallback((event) => {
						const newDate = new Date(date);
						newDate.setMonth(newDate.getMonth() - 1);
						onDateSelected(newDate);
					}, [date, onDateSelected])
				}>
					<CaretLeft />
				</button>
			</Tooltip>
			<Tooltip content={nextMonthName}>
				<button type={"button"} className={"next-month"} onClick={
					useCallback((event) => {
						const newDate = new Date(date);
						newDate.setMonth(newDate.getMonth() + 1);
						onDateSelected(newDate);
					}, [date, onDateSelected])
				}>
					<CaretRight />
				</button>
			</Tooltip>
		</div>
	);
}
