import React, {useCallback, useEffect, useMemo, useRef, useState} from "react";
import {classes, formatDate, Modify} from "../../Utils";
import {Float} from "../Floating/Float";
import {Datepicker} from "../Dates/Datepicker";

/**
 * A form input for a date with a datepicker.
 */
export function DatepickerInput(
	{
		children, className,
		value, onChange,
		// Properties to pass down.
		onKeyUp, onBlur,
		// Already set properties.
		type, placeholder,
		...props}: React.PropsWithChildren<Modify<React.InputHTMLAttributes<HTMLInputElement>, {
		/**
		 * The current date value.
		 */
		value?: Date|null;

		/**
		 * Called when picked date is changed.
		 * @param newDate The new date.
		 */
		onChange: (newDate: Date) => void;

		// Already set properties.
		type?: never;
		placeholder?: never;
	}>>): React.ReactElement
{
	// Date text state.
	const [dateText, setDateText] = useState("");

	// Update date text when date value has changed.
	useEffect(() => {
		if (value && value instanceof Date && !isNaN(value.getTime()))
			setDateText(formatDate(value));
	}, [value]);

	// Check if date is valid.
	const invalidDate = useMemo(() => !(value && value instanceof Date && !isNaN(value.getTime())) && dateText.length > 0, [value, dateText]);
	const dateValue = useMemo(() => invalidDate ? new Date() : (value ?? new Date()), [invalidDate, value]);

	const inputRef = useRef<HTMLInputElement>();

	/**
	 * Submit a new date from its raw text.
	 */
	const submitDate = useCallback((customDateText?: string) => {
		// Get the current date text status.
		const dateTextMatch = (customDateText ?? dateText).match(/^([0-9]?[0-9])\/([0-9]?[0-9])\/([0-9]?[0-9]?[0-9]?[0-9])$/);

		if (dateTextMatch)
		{
			// Parse day.
			let day = dateTextMatch[1];
			if (day.length < 2) day = "0" + day;
			if (parseInt(day) <= 0) day = "01";

			// Parse month.
			let month = dateTextMatch[2];
			if (month.length < 2) month = "0" + month;
			if (parseInt(month) <= 0) month = "01";

			// Parse year.
			let year = parseInt(dateTextMatch[3]);
			if (year < 100)
			{
				year += 1900;
				if ((new Date()).getFullYear() - year > 96) year += 100;
			}
			if (year < 1000) year += 1000;

			// Try to build the structurally valid date.
			const date = new Date(year + "-" + month + "-" + day);

			if (!isNaN(date.getTime()))
			{ // Date is valid, checking that it uses the right month (to fix the behavior when we go change 31/03 to 31/02, we want 28/02 or 29/02 and not 01/03).
				if ((date.getMonth() + 1) > parseInt(month))
				{ // Current date month is not valid, we're getting back to it.
					date.setDate(date.getDate() - 1);
				}
				if ((date.getMonth() + 1) < parseInt(month))
				{ // Current date month is not valid, we're getting back to it.
					date.setDate(date.getDate() + 1);
				}
			}

			// Try to keep original hours.
			date.setHours(value?.getHours() ?? 0, value?.getMinutes() ?? 0, value?.getSeconds() ?? 0, value?.getMilliseconds() ?? 0);

			// Set the structurally valid date.
			onChange?.(date);
		}
		else
			// No structurally valid date, removing it.
			onChange?.(null);
	}, [dateText, onChange]);

	return (
		<label className={classes("datepicker-input", invalidDate ? "error" : null, className)}
			     // Keeping focus on the input when something else in the label takes it.
			     onFocus={useCallback(() => inputRef.current?.focus(), [inputRef])}>
			{children}

			<Float mode={"focus"} className={"datepicker"} content={
				<Datepicker date={dateValue} onDateSelected={onChange} />
			}>
				<input type={"text"} ref={inputRef} placeholder={"DD/MM/AAAA"} value={dateText}
				       onChange={useCallback((event) => {
					       let dateText = event.currentTarget.value;

					       // Add first '/' if missing.
					       if (dateText.match(/^[0-9]{2}[^\/]*$/))
						       dateText = dateText.substring(0, 2) + "/" + dateText.substring(2);

					       // Add second '/' if missing.
					       if (dateText.match(/^[0-9]{2}\/[0-9]{2}[^\/]*$/))
						       dateText = dateText.substring(0, 5) + "/" + dateText.substring(5);

					       // Set the new date text.
					       setDateText(dateText);

					       // If a full date has been entered, reading it.
					       const fullDateMatch = dateText.match(/^([0-9]{2})\/([0-9]{2})\/([0-9]{4})$/);
					       if (fullDateMatch)
					       { // We have a structurally valid date, submitting it.
						       submitDate(dateText);
					       }
				       }, [submitDate])}
				       onKeyUp={useCallback((event) => {
					       if (event.key == "Enter")
						       // Submit date when enter is pressed.
						       submitDate();
								 return onKeyUp?.(event);
				       }, [submitDate, onKeyUp])}
					     onBlur={useCallback((event) => {
						     // Submit date when leaving form input.
								 submitDate();
								 return onBlur?.(event);
							 }, [submitDate, onBlur])}
					     {...props} />
			</Float>

			{ // The date is invalid, showing a subtext to say so.
				invalidDate && (
					<span className={"error subtext"}>Invalid date.</span>
				)
			}
		</label>
	);
}
